@@ -81,6 +81,8 @@ about that relationship.  RELATION may be C<undef> or the empty string, in
 which case the returned Lintian::Relation object is empty (always
 satisfied).
 
+=item parse_element
+
 =cut
 
 # The internal parser which converts a single package element of a
@@ -395,6 +397,8 @@ string.  If architecture restrictions should be ignored in RELATION,
 create a Lintian::Relation object with new_noarch() and pass that in as
 RELATION instead of the string.
 
+=item implies_element
+
 =cut
 
 # This internal function does the heavily lifting of comparing two
@@ -601,6 +605,10 @@ sub implies_element {
     return;
 }
 
+=item implies_array
+
+=cut
+
 # This internal function does the heavy of AND, OR, and NOT logic.  It expects
 # two references to arrays instead of an object and a relation.
 sub implies_array {
@@ -693,7 +701,7 @@ sub implies {
     if (ref($relation) ne 'Lintian::Relation') {
         $relation = Lintian::Relation->new($relation);
     }
-    return $self->implies_array($self, $relation);
+    return $self->implies_array($self, $relation) // 0;
 }
 
 =item implies_inverse(RELATION)
@@ -708,6 +716,8 @@ RELATION if it is a string.  If architecture restrictions should be
 ignored in RELATION, create a Lintian::Relation object with new_noarch()
 and pass that in as RELATION instead of the string.
 
+=item implies_element_inverse
+
 =cut
 
 # This internal function does the heavy lifting of inverse implication between
@@ -724,6 +734,10 @@ sub implies_element_inverse {
     return $result ? 0 : 1;
 }
 
+=item implies_array_inverse
+
+=cut
+
 # This internal function does the heavily lifting for AND, OR, and NOT
 # handling for inverse implications.  It takes two references to arrays and
 # returns true iff the falsehood of the second can be deduced from the truth
@@ -778,7 +792,7 @@ sub implies_inverse {
     if (ref($relation) ne 'Lintian::Relation') {
         $relation = Lintian::Relation->new($relation);
     }
-    return $self->implies_array_inverse($self, $relation);
+    return $self->implies_array_inverse($self, $relation) // 0;
 }
 
 =item unparse()
@@ -960,22 +974,22 @@ sub visit {
         my $against = $relation->[1];
         $against = $self->unparse($relation) if $flags & VISIT_PRED_FULL;
         local $_ = $against;
-        return $code->($against);
+        return scalar $code->($against);
     } elsif (($flags & VISIT_OR_CLAUSE_FULL) == VISIT_OR_CLAUSE_FULL
         and $rel_type eq 'OR') {
         my $against = $self->unparse($relation);
         local $_ = $against;
-        return $code->($against);
+        return scalar $code->($against);
     } elsif ($rel_type eq 'AND'
         or $rel_type eq 'OR'
         or $rel_type eq 'NOT') {
         for my $rel (@$relation[1 .. $#$relation]) {
-            my $ret = $self->visit($code, $flags, $rel);
+            my $ret = scalar $self->visit($code, $flags, $rel);
             if ($ret && ($flags & VISIT_STOP_FIRST_MATCH)) {
                 return $ret;
             }
         }
-        return;
+        return 0;
     }
 }
 
